'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof2 = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.default = build;

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _handlebars = require('handlebars');

var _handlebars2 = _interopRequireDefault(_handlebars);

var _isArray = require('lodash/isArray');

var _isArray2 = _interopRequireDefault(_isArray);

var _isNumber = require('lodash/isNumber');

var _isNumber2 = _interopRequireDefault(_isNumber);

var _isString = require('lodash/isString');

var _isString2 = _interopRequireDefault(_isString);

var _isBoolean = require('lodash/isBoolean');

var _isBoolean2 = _interopRequireDefault(_isBoolean);

var _isFunction = require('lodash/isFunction');

var _isFunction2 = _interopRequireDefault(_isFunction);

var _defaults = require('lodash/defaults');

var _defaults2 = _interopRequireDefault(_defaults);

var _cloneDeep = require('lodash/cloneDeep');

var _cloneDeep2 = _interopRequireDefault(_cloneDeep);

var _configuration = require('../share/configuration');

var _path3 = require('../share/path');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function build(modules, options, callback) {
  /* istanbul ignore next */
  if (!(0, _isFunction2.default)(callback)) {
    throw new Error('Callback is not provided');
  }

  options = (0, _defaults2.default)((0, _cloneDeep2.default)(options), {
    rootPath: _configuration.rootDir,
    logsPath: _configuration.logDir,
    distFile: _path2.default.join(options.rootPath || _configuration.rootDir, 'vhosts/nginx.conf'),
    template: _path2.default.join(_configuration.execDir, 'libs/templates/vhosts/nginx.conf.hbs'),
    useHttps: (0, _isBoolean2.default)(options.useHttps) ? options.useHttps : false
  });

  /* istanbul ignore next */
  if (!_fsExtra2.default.existsSync(options.template)) {
    callback(new Error('Template \'' + options.template + '\' is not exists.'));
    return;
  }

  var template = _fsExtra2.default.readFileSync(options.template, 'utf-8');
  var compile = _handlebars2.default.compile(template);

  modules = (0, _cloneDeep2.default)(modules);

  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = modules[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var module = _step.value;

      /* istanbul ignore next */
      if (!module.domain) {
        callback(new Error('Domain is not provided'));
        return;
      }

      if (module.type === 'module') {
        /* istanbul ignore next */
        if (!module.entry) {
          callback(new Error('Entry is not provided'));
          return;
        }

        var entry = (0, _path3.absolute)(module.entry, options.rootPath);

        /* istanbul ignore next */
        if (!_fsExtra2.default.existsSync(entry)) {
          callback(new Error('Entry file ' + entry + ' is not found'));
          return;
        }

        var folder = _path2.default.dirname(entry);
        module.entry = _path2.default.basename(folder) + '.html';

        /* istanbul ignore next */
        if (!((0, _isString2.default)(module.proxy) && /^(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])\.(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])\.(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])\.(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])$/.exec(module.proxy))) {
          callback(new Error('Proxy is not provided or invalid. (Proxy is a ip address, eg: 127.0.0.1)'));
          return;
        }

        /* istanbul ignore next */
        if (!(0, _isNumber2.default)(module.port)) {
          callback(new Error('port is not provided or invalid. (port must be a port number)'));
          return;
        }
      }

      if (module.useHttps === true) {
        var certFile = (0, _path3.absolute)(module.certFile, options.rootPath);
        /* istanbul ignore next */
        if (!certFile) {
          callback(new Error('CertFile is not provided when use https'));
          return;
        }

        /* istanbul ignore next */
        if (!_fsExtra2.default.existsSync(certFile)) {
          callback(new Error('CertFile ' + certFile + ' is not found'));
          return;
        }

        var certKey = (0, _path3.absolute)(module.certKey, options.rootPath);
        /* istanbul ignore next */
        if (!certKey) {
          callback(new Error('CertKey is not provided when use https'));
          return;
        }

        /* istanbul ignore next */
        if (!_fsExtra2.default.existsSync(certKey)) {
          callback(new Error('CertKey ' + certKey + ' is not found'));
          return;
        }
      }
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  _fsExtra2.default.ensureDirSync(options.logsPath);

  var source = compile({
    rootPath: _configuration.distDir,
    logsPath: options.logsPath,
    modules: modules
  });

  _fsExtra2.default.ensureDir(options.distFile.replace(_path2.default.basename(options.distFile), ''));
  _fsExtra2.default.writeFile(options.distFile, source, function (error) {
    /* istanbul ignore next */
    if (error) {
      callback(error);
      return;
    }

    callback(null, { file: options.distFile, modules: modules });
  });
}

/**
 * compare values
 * @param  {String} lvalue   first value
 * @param  {String} operator operator symbol
 * @param  {String} rvalue   second value
 * @param  {Object} options
 */
function compare(lvalue, operator, rvalue, options) {
  var operators = void 0;
  var result = void 0;

  /* istanbul ignore next */
  if (arguments.length < 3) {
    throw new Error('Handlerbars Helper "compare" needs 2 parameters');
  }

  if (undefined === options) {
    options = rvalue;
    rvalue = operator;
    operator = '===';
  }

  operators = {
    '==': function _(l, r) {
      /* eslint eqeqeq:off */
      return l == r;
    },
    '===': function _(l, r) {
      return l === r;
    },
    '!=': function _(l, r) {
      /* eslint eqeqeq:off */
      return l != r;
    },
    '!==': function _(l, r) {
      return l !== r;
    },
    '<': function _(l, r) {
      return l < r;
    },
    '>': function _(l, r) {
      return l > r;
    },
    '<=': function _(l, r) {
      return l <= r;
    },
    '>=': function _(l, r) {
      return l >= r;
    },
    'typeof': function _typeof(l, r) {
      /* eslint eqeqeq:off */
      /* eslint valid-typeof: off */
      return (typeof l === 'undefined' ? 'undefined' : _typeof2(l)) == r;
    }
  };

  /* istanbul ignore next */
  if (!operators[operator]) {
    throw new Error('Handlerbars Helper \'compare\' doesn\'t know the operator ' + operator);
  }

  result = operators[operator](lvalue, rvalue);
  return result ? options.fn(this) : options.inverse(this);
}

/**
 * separate array
 * @param  {String|Array} value     input value
 * @param  {String}       separator separate symbol
 * @return {String}
 */
function separate(value, separator) {
  if (arguments.length < 3) {
    separator = ' ';
  }

  if ((0, _isString2.default)(value)) {
    return value;
  }

  if ((0, _isArray2.default)(value)) {
    return value.join(separator);
  }
}

/**
 * Register Handlebars helpers
 * @docs: http://handlebarsjs.com/block_helpers.html
 */

/**
 * Compare number and type-equals with variables
 */
_handlebars2.default.registerHelper('compare', compare);

/**
 * Separate Array to some string.
 * [value1, value2, value3] => 'value1 value2 value3'
 */
_handlebars2.default.registerHelper('separate', separate);