'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (config) {
  var karmaConf = {
    basePath: _configuration.rootDir,
    browsers: ['PhantomJS'],
    frameworks: ['mocha', 'chai', 'sinon'],
    files: [{
      pattern: './test/e2e/*.spec.js',
      watched: false
    }],
    client: {
      chai: {
        includeStack: true
      }
    },
    preprocessors: {
      './test/e2e/**/*.spec.js': ['webpack', 'sourcemap']
    },
    reporters: ['mocha'],
    webpack: _webpackUnitestConfig2.default,
    webpackMiddleware: {
      noInfo: false,
      stats: true
    },
    /**
     * in empty test folder, it will return
     * status 1 and throw error.
     * set 'failOnEmptyTestSuite' to false
     * will resolve this problem.
     */
    failOnEmptyTestSuite: false,
    autoWatch: false,
    singleRun: true,
    colors: true,
    logLevel: config.LOG_INFO,
    plugins: ['karma-phantomjs-launcher', 'karma-webpack', 'karma-chai', 'karma-sinon', 'karma-mocha', 'karma-mocha-reporter', 'karma-coverage-istanbul-reporter', 'karma-sourcemap-loader']
  };

  if (process.env.COVERAGE) {
    karmaConf.reporters = ['mocha', 'coverage-istanbul'];

    karmaConf.coverageIstanbulReporter = {
      reports: ['lcov', 'text-summary'],
      fixWebpackSourcePaths: true
    };

    karmaConf.webpack = (0, _webpackMerge2.default)(_webpackUnitestConfig2.default, {
      module: {
        rules: [{
          test: /\.js$/,
          enforce: 'post',
          include: _webpackCommonConfig.ResolveModules,
          exclude: [/node_modules/, /\.spec\.js$/, _path2.default.join(_configuration.srcDir, './assets/sprites/svg')],
          use: [{
            loader: 'istanbul-instrumenter-loader',
            query: {
              esModules: true
            }
          }]
        }]
      }
    });
  }

  config.set(karmaConf);
};

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _filter = require('lodash/filter');

var _filter2 = _interopRequireDefault(_filter);

var _indexOf = require('lodash/indexOf');

var _indexOf2 = _interopRequireDefault(_indexOf);

var _webpackMerge = require('webpack-merge');

var _webpackMerge2 = _interopRequireDefault(_webpackMerge);

var _webpackCommonConfig = require('./webpack.common.config.babel');

var _webpackUnitestConfig = require('./webpack.unitest.config.babel');

var _webpackUnitestConfig2 = _interopRequireDefault(_webpackUnitestConfig);

var _configuration = require('../share/configuration');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

_webpackUnitestConfig2.default.entry = {
  index: ['babel-polyfill']

  /**
   * Filter CommonsChunkPlugin
   * docs: https://github.com/webpack-contrib/karma-webpack/issues/24
   */
};_webpackUnitestConfig2.default.plugins = (0, _filter2.default)(_webpackUnitestConfig2.default.plugins, function (plugin) {
  var index = (0, _indexOf2.default)(['CommonsChunkPlugin', 'FaviconsWebpackPlugin', 'HtmlWebpackPlugin'], plugin.constructor.name);
  return index === -1;
});