'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Rules = exports.CallAfter = exports.Injector = exports.Plugins = exports.ResolveModules = exports.Entries = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

exports.generateEnteries = generateEnteries;
exports.generateFavicons = generateFavicons;
exports.generateSprites = generateSprites;
exports.generateSVGSprites = generateSVGSprites;
exports.WithDonePlugin = WithDonePlugin;
exports.InjectScriptPlugin = InjectScriptPlugin;

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _map = require('lodash/map');

var _map2 = _interopRequireDefault(_map);

var _assign = require('lodash/assign');

var _assign2 = _interopRequireDefault(_assign);

var _without = require('lodash/without');

var _without2 = _interopRequireDefault(_without);

var _isArray = require('lodash/isArray');

var _isArray2 = _interopRequireDefault(_isArray);

var _indexOf = require('lodash/indexOf');

var _indexOf2 = _interopRequireDefault(_indexOf);

var _forEach = require('lodash/forEach');

var _forEach2 = _interopRequireDefault(_forEach);

var _isObject = require('lodash/isObject');

var _isObject2 = _interopRequireDefault(_isObject);

var _isString = require('lodash/isString');

var _isString2 = _interopRequireDefault(_isString);

var _isFunction = require('lodash/isFunction');

var _isFunction2 = _interopRequireDefault(_isFunction);

var _filter = require('lodash/filter');

var _filter2 = _interopRequireDefault(_filter);

var _defaults = require('lodash/defaults');

var _defaults2 = _interopRequireDefault(_defaults);

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _cleanWebpackPlugin = require('clean-webpack-plugin');

var _cleanWebpackPlugin2 = _interopRequireDefault(_cleanWebpackPlugin);

var _spritesheetTemplates = require('spritesheet-templates');

var _spritesheetTemplates2 = _interopRequireDefault(_spritesheetTemplates);

var _webpackSpritesmith = require('webpack-spritesmith');

var _webpackSpritesmith2 = _interopRequireDefault(_webpackSpritesmith);

var _webpackSvgstorePlugin = require('webpack-svgstore-plugin');

var _webpackSvgstorePlugin2 = _interopRequireDefault(_webpackSvgstorePlugin);

var _faviconsWebpackPlugin = require('favicons-webpack-plugin');

var _faviconsWebpackPlugin2 = _interopRequireDefault(_faviconsWebpackPlugin);

var _htmlWebpackPlugin = require('html-webpack-plugin');

var _htmlWebpackPlugin2 = _interopRequireDefault(_htmlWebpackPlugin);

var _autoprefixer = require('autoprefixer');

var _autoprefixer2 = _interopRequireDefault(_autoprefixer);

var _extractTextWebpackPlugin = require('extract-text-webpack-plugin');

var _extractTextWebpackPlugin2 = _interopRequireDefault(_extractTextWebpackPlugin);

var _copyWebpackPlugin = require('copy-webpack-plugin');

var _copyWebpackPlugin2 = _interopRequireDefault(_copyWebpackPlugin);

var _configuration = require('../share/configuration');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var DefinePlugin = _webpack2.default.DefinePlugin;
var CommonsChunkPlugin = _webpack2.default.optimize.CommonsChunkPlugin;
var GlobalVariables = (0, _defaults2.default)({ publicPath: JSON.stringify(_configuration.publicPath) }, _configuration.variables);

/**
 * Entries definitions
 */
var Entries = exports.Entries = {};

/**
 * reolve path definitions
 */
var ResolveModules = exports.ResolveModules = [_path2.default.join(_configuration.execDir, 'node_modules'), _path2.default.join(_configuration.rootDir, 'node_modules'), _configuration.srcDir, _configuration.tmpDir];

/**
 * Plugins definitions
 */
var Plugins = exports.Plugins = [
/**
 * Define some global variables
 */
new DefinePlugin(GlobalVariables),
/**
 * Extract common modules
 * to reduce code duplication
 */
new CommonsChunkPlugin({
  name: 'vendor',
  minChunks: function minChunks(module) {
    return (/node_modules/.test(module.resource)
    );
  }
}),

/**
 * Extract style file
 * Inline styles can be externally optimized for loading
 */
new _extractTextWebpackPlugin2.default({
  filename: 'styles/[name].[contenthash].css',
  allChunks: true
}),

/**
 * Copy files
 */
new _copyWebpackPlugin2.default([{
  from: _path2.default.join(_configuration.rootDir, 'assets/panels/**'),
  to: _path2.default.join(_configuration.distDir, 'assets/panels/'),
  flatten: true
}]),

/**
 * Clean generate folders
 * run it first to reset the project.
 */
new _cleanWebpackPlugin2.default([_configuration.tmpDir, _configuration.distDir], {
  root: '/',
  verbose: true,
  dry: false
})];

var Injector = exports.Injector = InjectScriptPlugin(Plugins);
var CallAfter = exports.CallAfter = WithDonePlugin(Plugins);

/**
 * Generate some compile tasks
 */
generateEnteries(Plugins, Entries);
generateFavicons(Plugins);
generateSVGSprites(Plugins, Entries);
var spriteGenerated = generateSprites(Plugins);

/**
 * loader and rules definitions
 */
var Rules = exports.Rules = [{
  test: /\.html$/,
  use: [{
    loader: 'html-loader',
    options: {
      attrs: ['img:src', 'img:ng-src']
    }
  }]
}, {
  test: /\.pug$/,
  use: [{
    loader: 'pug-loader'
  }]
},
/**
 * As Jade/Pug will use require() to load public style
 * like bootstrap.css, so that we must provider a loader
 * to load the file.
 * At the same time, `ExtractTextPlugin` plugin do not
 * match .css file, because it will throw an error to
 * tell you no loader can load this file.
 *
 * Error:
 *   Module build failed:
 *   Error: "extract-text-webpack-plugin" loader is used
 *   without the corresponding plugin, refer to
 *   https://github.com/webpack/extract-text-webpack-plugin
 *   for the usage example
 */
{
  test: /\.css$/,
  use: {
    loader: 'url-loader',
    options: {
      limit: 10000,
      name: 'styles/[name].[hash].css'
    }
  }
},
/**
 * docs:
 * - https://github.com/webpack-contrib/extract-text-webpack-plugin/issues/263
 */
{
  test: /\.(sass|scss)$/,
  use: _extractTextWebpackPlugin2.default.extract({
    fallback: 'style-loader',
    use: [{
      loader: 'css-loader',
      options: {
        minimize: true
      }
    }, {
      loader: 'sass-loader',
      options: {
        includePaths: ResolveModules,
        data: [spriteGenerated ? '@import "sprites";' : ''].join('\n')
      }
    }, {
      loader: 'postcss-loader',
      options: {
        plugins: [(0, _autoprefixer2.default)({
          browsers: ['last 10 version', 'ie >= 9']
        })]
      }
    }]
  })
}, {
  test: /\.js$/,
  use: [{
    loader: 'ng-annotate-loader'
  },
  /**
   * babel@6.0.0 break the .babelrc file
   * so configure presets below
   * docs:
   * - https://github.com/babel/babel-loader/issues/166
   */
  {
    loader: 'babel-loader',
    options: {
      plugins: [require.resolve('babel-plugin-transform-decorators-legacy'), require.resolve('babel-plugin-transform-export-extensions')],
      presets: [require.resolve('babel-preset-es2015'), require.resolve('babel-preset-stage-0')]
    }
  }],
  exclude: [/node_modules/]
},
/**
 * 少于 10K 图片用 base64
 * url-loader 依赖 file-loader
 */
{
  test: /\.(jpe?g|png|gif)$/i,
  use: [{
    loader: 'url-loader',
    options: {
      limit: 10000,
      name: 'panels/[name].[hash].[ext]'
    }
  }]
}];

/**
 * Webpack Setting
 */
exports.default = {
  entry: Entries,
  output: {
    path: _configuration.distDir,
    publicPath: _configuration.publicPath,
    filename: '[name].js'
  },
  module: {
    rules: Rules
  },
  resolve: {
    modules: ResolveModules
  },
  resolveLoader: {
    modules: ResolveModules
  },
  plugins: Plugins

  /**
   * Auto generate entries
   * Generate entries dependent on folder (src/entry/{folder})
   * And entry js file must be named 'index.js'
   */
};
function generateEnteries(plugins, entries) {
  if (!(0, _isArray2.default)(plugins)) {
    throw new Error('Parameter plugins must be a array.');
  }

  if (!(0, _isObject2.default)(entries)) {
    throw new Error('Parameter entries must be a object.');
  }

  var modules = [];
  (0, _forEach2.default)(_configuration.modules, function (module) {
    if (!module.entry) {
      return;
    }

    var file = _path2.default.join(_configuration.rootDir, module.entry);
    if (!_fsExtra2.default.existsSync(file)) {
      throw new Error('Entry file ' + module.entry + ' is not found');
    }

    var dir = _path2.default.dirname(file);
    var name = _path2.default.basename(dir);

    modules.push({ file: file, dir: dir, name: name });
  });

  if (modules.length === 0) {
    throw new Error('Entry is not found');
  }

  var names = (0, _map2.default)(modules, 'name');
  (0, _forEach2.default)(modules, function (_ref) {
    var file = _ref.file,
        dir = _ref.dir,
        name = _ref.name;

    entries[name] = ['babel-polyfill', file];

    /**
     * reanme entry html
     */
    var options = {
      filename: _path2.default.join(_configuration.distDir, name + '.html')

      /**
       * use template when then template file exists
       */
    };var templateFile = _path2.default.join(dir, 'index.pug');
    _fsExtra2.default.existsSync(templateFile) && (0, _assign2.default)(options, { template: templateFile });

    /**
     * clean other static resources
     */
    var excludeChunks = (0, _without2.default)(names, name);
    (0, _assign2.default)(options, { excludeChunks: excludeChunks });

    var plugin = new _htmlWebpackPlugin2.default(options);
    plugins.push(plugin);

    return true;
  });
}

/**
 * auto split logo task
 * if logo file not exists, this task will not be executed.
 */
function generateFavicons(plugins) {
  if (!(0, _isArray2.default)(plugins)) {
    throw new Error('Parameter plugins must be a array.');
  }

  var logoFile = _path2.default.join(_configuration.srcDir, './assets/panels/logo.png');
  if (_fsExtra2.default.existsSync(logoFile)) {
    var statsFile = 'favicon/iconstats.json';
    var plugin = new _faviconsWebpackPlugin2.default({
      logo: logoFile,
      prefix: 'favicon/[hash]/',
      emitStats: true,
      statsFilename: statsFile,
      persistentCache: true,
      inject: true,
      background: '#fff',
      icons: {
        android: true,
        appleIcon: true,
        appleStartup: false,
        coast: false,
        favicons: true,
        firefox: true,
        opengraph: false,
        twitter: false,
        yandex: false,
        windows: false
      }
    });

    plugins.push(plugin);

    /**
     * after compile, it will generate 'favicon.ico' file
     * and copy it to the root path.
     */
    CallAfter.add(function () {
      var sourceFile = _path2.default.join(_configuration.distDir, statsFile);
      if (!_fsExtra2.default.existsSync(sourceFile)) {
        return;
      }

      var stats = _fsExtra2.default.readJsonSync(sourceFile);
      var favFile = _path2.default.join(_configuration.distDir, stats.outputFilePrefix, 'favicon.ico');
      var faviconFile = _path2.default.join(_configuration.distDir, 'favicon.ico');

      _fsExtra2.default.copySync(favFile, faviconFile);
    });

    return true;
  }

  return false;
}

/**
 * Auto concat sprite image
 * if the sprite folder (src/assets/sprites/images/) not exists,
 * this task will not be excuted.
 */
function generateSprites(plugins) {
  if (!(0, _isArray2.default)(plugins)) {
    throw new Error('Parameter plugins must be a array.');
  }

  var spriteDir = _path2.default.join(_configuration.srcDir, './assets/sprites/images');
  var files = _fsExtra2.default.readdirSync(spriteDir);
  var images = (0, _filter2.default)(files, function (file) {
    return (/(\.png|\.gif|\.jpg)$/.test(file)
    );
  });

  if (images.length === 0) {
    return false;
  }

  var spriteTemplate = _path2.default.join(spriteDir, 'sprite.scss.template.handlebars');
  if (_fsExtra2.default.existsSync(spriteDir) && _fsExtra2.default.lstatSync(spriteDir).isDirectory() && _fsExtra2.default.existsSync(spriteTemplate)) {
    var source = _fsExtra2.default.readFileSync(spriteTemplate, 'utf8');
    _spritesheetTemplates2.default.addHandlebarsTemplate('spriteScssTemplate', source);

    var plugin = new _webpackSpritesmith2.default({
      src: {
        cwd: spriteDir,
        glob: '**/*.{png,gif,jpg}'
      },
      target: {
        image: _path2.default.join(__dirname, '.build/sprites.png'),
        css: [[_path2.default.join(__dirname, '.build/sprites.scss'), {
          format: 'spriteScssTemplate'
        }]]
      },
      apiOptions: {
        cssImageRef: '~sprites.png'
      },
      spritesmithOptions: {
        functions: true,
        padding: 10
      }
    });

    plugins.push(plugin);

    return true;
  }

  return false;
}

/**
 * Auto concat svg sprite image
 * if the sprite folder (src/assets/sprites/svg/) not exists,
 * this task will not be excuted.
 */
function generateSVGSprites(plugins, entries) {
  if (!(0, _isArray2.default)(plugins)) {
    throw new Error('Parameter plugins must be a array.');
  }

  var spriteDir = _path2.default.join(_configuration.srcDir, './assets/sprites/svg');
  var spriteTemplate = _path2.default.join(spriteDir, 'svgstore.config.js');

  if (_fsExtra2.default.existsSync(spriteDir) && _fsExtra2.default.lstatSync(spriteDir).isDirectory() && _fsExtra2.default.existsSync(spriteTemplate)) {
    var files = _fsExtra2.default.readdirSync(spriteDir);
    var images = (0, _filter2.default)(files, function (file) {
      return (/\.svg$/.test(file)
      );
    });

    if (images.length === 0) {
      return false;
    }

    (0, _assign2.default)(Entries, { svgstore: spriteTemplate });

    var plugin = new _webpackSvgstorePlugin2.default({
      prefix: 'sp-svg-',
      svgoOptions: {
        plugins: [{ removeComments: true }, { removeMetadata: true }, { removeTitle: true }, { removeDesc: true }, { removeUselessDefs: true }, { removeXMLNS: true }, { minifyStyles: true }, { cleanupIDs: true }, { removeEmptyText: true }, { convertColors: true }, { convertPathData: true }, { convertTransform: true }, { removeUnknownsAndDefaults: true }, { removeUnusedNS: true },
        /**
         * svg in webkit old browser, it not support use (reference)
         * it must use '<use xlink:href="url#id"></use>'
         * and because svgo(https://github.com/svg/svgo) do not set
         * 'xmlns:xlink="http://www.w3.org/1999/xlink"', so it make
         * origin svg content with use tag lack 'namespace' 'prefix',
         * and it make svg display success.
         *
         * SVG 在 webkit 低版本浏览器中不支持内联 use
         * 必须使用 <use xlink:href="url#id"></use>
         * 又因为 svgo 并没有设置 xmlns:xlink="http://www.w3.org/1999/xlink"
         * 因此会使原本 svg 内含有 use 缺少 namespace prefix 的问题, 导致没法兼容加载,
         * 导致外部不能成功导入
         *
         * Error Code:
         * This page contains the following errors:
         * error on line 1 at column 15734: Namespace prefix xlink for href on use is not defined
         * Below is a rendering of the page up to the first error.
         *
         * Browser: Chrome 48.0.2564.23:
         * Mozilla/5.0 (Linux; Android 6.0; Nexus 5 Build/MRA58N)
         * AppleWebKit/537.36 (KHTML, like Gecko)
         * Chrome/48.0.2564.23
         * Mobile Safari/537.36
         * wechatdevtools/0.7.0
         * MicroMessenger/6.3.22
         * webview/0
         *
         * Docs : https://github.com/svg/svgo/blob/master/docs/how-it-works/en.md#3-plugins
         * API  : https://github.com/svg/svgo/blob/master/docs/how-it-works/en.md#32-api
         */
        {
          downwardCompatible: {
            type: 'perItem',
            active: true,
            description: 'Backward compatibility, <use> add attribute xmlns:xlink="http://www.w3.org/1999/xlink"',
            params: {},
            fn: function fn(item) {
              if (item.isElem('use') && !hasAttr(item, 'xmlns:xlink')) {
                setAttr(item, 'xmlns:xlink', 'http://www.w3.org/1999/xlink');
              }
            }
          }
        }]
      }
    });

    plugins.push(plugin);
    return true;
  }

  return false;

  function hasAttr(item, name) {
    var _name$split = name.split(':'),
        _name$split2 = _slicedToArray(_name$split, 2),
        prefix = _name$split2[0],
        local = _name$split2[1];

    return (0, _indexOf2.default)(item.attrs, { name: name, prefix: prefix, local: local }) !== -1;
  }

  function setAttr(item, name, value) {
    var _name$split3 = name.split(':'),
        _name$split4 = _slicedToArray(_name$split3, 2),
        prefix = _name$split4[0],
        local = _name$split4[1];

    item.attrs[name] = { value: value, name: name, prefix: prefix, local: local };
  }
}

/**
 * Callback after webpack excutes
 */
function WithDonePlugin(plugins) {
  if (!(0, _isArray2.default)(plugins)) {
    throw new Error('Parameter plugins must be a array.');
  }

  var instance = {
    _callbacks: [],
    add: function add(callback) {
      (0, _isFunction2.default)(callback) && this._callbacks.push(callback);
    },
    done: function done() {
      (0, _forEach2.default)(this._callbacks, function (callback) {
        return callback();
      });
    }
  };

  plugins.push({
    apply: function apply(compiler) {
      compiler.plugin('done', instance.done.bind(instance));
    }
  });

  return instance;
}

/**
 * Inject script to entry html file
 */
function InjectScriptPlugin(plugins) {
  if (!(0, _isArray2.default)(plugins)) {
    throw new Error('Parameter plugins must be a array.');
  }

  var instance = {
    _injector: [],
    _callbacks: [],
    inject: function inject(source) {
      if (!(0, _isString2.default)(source)) {
        return false;
      }

      var hash = mkhash(source);
      if ((0, _indexOf2.default)(this._injector, { hash: hash }) !== -1) {
        return false;
      }

      var script = '!(function () {\n        var id = \'webpack-' + hash + '\';\n        if (\'undefined\' === typeof window || document.getElementById(id)) {\n          return;\n        }\n\n        var node = document.createElement(\'script\');\n        node.id = id;\n        node.innerHTML = \'' + source + '\';\n\n        document.head.appendChild(node);\n      })();';

      this._injector.push({ hash: hash, script: script });
    },
    after: function after(callback) {
      (0, _isFunction2.default)(callback) && this._callbacks.push(callback);
    }
  };

  plugins.push({
    autoloadScript: function autoloadScript() {
      var scripts = [];
      (0, _forEach2.default)(instance._injector, function (injector) {
        return scripts.push(injector.script);
      });
      return scripts.join('\n');
    },
    scriptTag: function scriptTag(source) {
      var injector = this.autoloadScript();
      return injector + source;
    },
    applyCompilation: function applyCompilation(compilation) {
      compilation.mainTemplate.plugin('startup', this.scriptTag.bind(this));
    },
    applyDone: function applyDone() {
      (0, _forEach2.default)(instance._callbacks, function (injector) {
        return injector();
      });
      instance._callbacks.splice(0);
    },
    apply: function apply(compiler) {
      /**
       * sometimes it will trigger twice or more,
       * the core-code just exec once, see below function autoloadScript.
       */
      compiler.plugin('compilation', this.applyCompilation.bind(this));
      compiler.plugin('done', this.applyDone.bind(this));
    }
  });

  return instance;
}

/**
 * make hash code
 * See: http://stackoverflow.com/questions/7616461/generate-a-hash-from-string-in-javascript-jquery?answertab=active#tab-top
 */
function mkhash(string) {
  string = JSON.stringify(string);

  var hash = 0;
  if (string.length === 0) {
    return hash;
  }

  for (var i = 0, l = string.length; i < l; i++) {
    var chr = string.charCodeAt(i);
    hash = (hash << 5) - hash + chr;

    /**
     * Convert to 32bit integer
     */
    hash |= 0;
  }

  return hash;
}