'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.formatBytes = formatBytes;
exports.trace = trace;
exports.printStats = printStats;

var _isEmpty = require('lodash/isEmpty');

var _isEmpty2 = _interopRequireDefault(_isEmpty);

var _indexOf = require('lodash/indexOf');

var _indexOf2 = _interopRequireDefault(_indexOf);

var _defaultsDeep = require('lodash/defaultsDeep');

var _defaultsDeep2 = _interopRequireDefault(_defaultsDeep);

var _colors = require('colors');

var _colors2 = _interopRequireDefault(_colors);

var _columnify = require('columnify');

var _columnify2 = _interopRequireDefault(_columnify);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var ingoreTrace = (0, _indexOf2.default)(process.argv, '--quiet') === -1;

/**
 * format size by unit
 * @param  {Number} bytes    size
 * @param  {Number} decimals
 * @return {String}
 */
function formatBytes(bytes, decimals) {
  if (bytes === 0) {
    return '0 Bytes';
  }

  var k = 1000;
  var dm = decimals + 1 || 3;
  var sizes = ['Bytes', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'];
  var i = Math.floor(Math.log(bytes) / Math.log(k));

  return parseFloat((bytes / Math.pow(k, i)).toFixed(dm)) + ' ' + sizes[i];
}

/**
 * print stats
 * @param  {Object} options setting
 */
function trace(message) {
  ingoreTrace && console.log(message);
}

/**
 * Print results
 * @param  {Array}  stats   result set
 * @param  {Object} options columnify setting
 */
function printStats(stats, options) {
  /* istanbul ignore if */
  if ((0, _isEmpty2.default)(stats)) {
    trace(_colors2.default.yellow('Generate completed but nothing to be generated.'));
    return false;
  }

  options = (0, _defaultsDeep2.default)(options, {
    headingTransform: function headingTransform(heading) {
      return _colors2.default.white.bold(heading.charAt(0).toUpperCase() + heading.slice(1));
    },
    config: {
      domain: {
        align: 'right',
        dataTransform: function dataTransform(domain) {
          return _colors2.default.green.bold(domain);
        }
      },
      proxy: {
        align: 'right'
      },
      entry: {
        align: 'left',
        dataTransform: function dataTransform(entry) {
          return _colors2.default.green.bold(entry);
        }
      },
      port: {
        align: 'right'
      }
    }
  });

  /* eslint no-console:off */
  trace((0, _columnify2.default)(stats, options) + '\n');
  return true;
}